/**
 * @typedef Env
 *
 * @typedef DryRunResult
 * @property {any} Output
 * @property {any[]} Messages
 * @property {any[]} Spawns
 * @property {any} [Error]
 *
 * @typedef MessageInput
 * @property {string} process
 * @property {any} [data]
 * @property {{ name: string, value: string }[]} [tags]
 * @property {string} [anchor]
 * @property {string} [Id]
 * @property {string} [Owner]
 *
 * @callback DryRun
 * @param {MessageInput & Object.<string, *>} msg
 * @return {Promise<DryRunResult>}
 *
 * @param {Env} env
 * @returns {DryRun}
 */
export function dryrunWith(env: any): DryRun;
export type Env = any;
export type DryRunResult = {
    Output: any;
    Messages: any[];
    Spawns: any[];
    Error?: any;
};
export type MessageInput = {
    process: string;
    data?: any;
    tags?: {
        name: string;
        value: string;
    }[];
    anchor?: string;
    Id?: string;
    Owner?: string;
};
export type DryRun = (msg: MessageInput & {
    [x: string]: any;
}) => Promise<DryRunResult>;
